#---------------------------Marsh expansion model -----------------------------------
# Seed-based marsh expansion in the tidal flats next to the marsh
# by Zhenchang Zhu and Johan van de Kopel @ Sept.29th, 2017

# First setup of the model
remove(list=ls()) # Remove all variables from memory
#install.packages("simecol")
require("simecol")

# --- Parameters --------------------------------------------------------------

resolution=0.1 # size of each cell, unit => m
length1=100 # along shore lengh of the landscpae, unit => m
length2=100 # cross shore length of the landscape 

n1=length1/resolution # number of rows
n2=length2/resolution # number of coloumns 

Tau=0.01 # TIme-averaged wave-induced bed shear stress 
SDC=0.1 # Seedling density coefficient 
Ks_max = 25/resolution #  Maximum value of the half satuation constant for seedling density decay with distance 
step_max= 2/resolution # maximum value of the clonal growth rate
EndTime=5

# Calculate seedling establishment capacity 
SP=exp(-102.6*Tau) #  Seed persistence as a funcion of wave-induced bed shear stress 
PSD=SP*SDC # Peak seedling density (Averaged number of seedlings per cell)

# --- Seedling establishment function definition ------------------------------
SeedlingEstablishment = function(landscape, PSD, Ks)
{       
        n1=dim(landscape)[1]
        n2=dim(landscape)[2]
        Closed=which(colSums(landscape==2)>=0.9*n1) # which rows are (almost) closed
        border = Closed[length(Closed)]  # find the border of the closed vegetation
        border = max(0, border) # if no border, then make sure border = 0
        
        if (border==n2){
                landscape=matrix(nrow=n1,ncol=n2, data=0)
        }
        else {
                distance=1:(n2-border)             # distance to the marsh edge 
                SD=PSD*Ks/(Ks+distance);              # Seedling density at a given distance
                Seedlingnum=round(n1*SD)     # number of seedling cells at a given distance (x coordinate)
                
                
                for (i in (border+1):n2)           # loop for each distance 
                {
                        sn=min(n1, Seedlingnum[i-border])       # Number of seedlings at that distance 
                        Seed_location=sample(n1,sn)     # Select the row numbers for the seedlings, from the randomized x coordinates 
                        landscape[Seed_location,i]=1     # Change the value to 1 for the cells that occupied by seedlings. 
                } 
        }
        return (landscape==1)*1              # return the new landscape 
        
}
# --- Initialization ----------------------------------------------------------

R=matrix(nrow=n1, ncol=n2, data=runif(n1*n2)) # a field of random values between 0 and 1. 
Cells=matrix(nrow=n1,ncol=n2, data=0)       # generate a landscape 

Cell_Plant=matrix(ncol=1, nrow=EndTime, data=0)

# --- Initilize the figure ----------------------------------------------------

WinWidth = 800
WinHeight = 800
DPI = 144

## Open a graphics window (Darwin stands for a Mac computer)
if (Sys.info()["sysname"]=="Darwin"){
        quartz(width=WinWidth/DPI, height=WinHeight/DPI, 
               title="Zhu's model")
} else
        windows(width = WinWidth/DPI, height = WinHeight/DPI,
                title="Zhu's model")

par(mfrow=c(1,1), mar=c(5, 5, 5, 5))


for (t in 1:EndTime){     
        Seedlings = matrix(nrow=n1,ncol=n2, data=0)
        if (sample(x=c(0,1),size=1,replace=F, prob=NULL)) # ramdomly determine whethere it is a bad year
        { 
                Ks=sample(x=10:Ks_max,size=1,replace=F, prob=NULL)
                Seedlings = SeedlingEstablishment(Cells,PSD, Ks)     
        }
        # else { 
              #  Seedlings = matrix(nrow=n1,ncol=n2, data=0)
             #   }
        
        # Defining a circular growth neighborhood
        step=sample(x=1:step_max, size=1, replace=F, prob=NULL)
        XY=matrix(nrow=step*2+1, ncol=step*2+1, data=seq(-step,step),(step*2)+1)
        z=sqrt(XY^2+t(XY)^2)<=step; z[step+1,step+1]=0
        GrowthNeighbourhood=z
        
        ## For each cell, calculate the number of vegetated neighbours
        Sumveg=matrix(nrow=n1,ncol=n2, neighbours(x=Cells,state=2,wdist=GrowthNeighbourhood))
        C=Sumveg/4 # clonal growth capacity. 
                   # Transition probability from a empty cell to a vegetated cell.
                   # If there are 4 or more vegetated cells, 
                   # in the neighbourhood, the clonal growth probability will be 1
        
        Growth_Clone=(Seedlings==0)&(Cells==0)&(R<=C)
        Growth_Clone=2*Growth_Clone
        
        # Vegetation growth due to seedlings; Each seedling cell (1) becomes a fully vegetated cell (2) 
        Seedlings=(Seedlings==1)&(Cells==0); #Exclude the overlap of seedling cells and vegetated cells 
        Growth_Seedling = Seedlings*2;
        
        # The new cell states 
        Cells=Cells+Growth_Clone+Growth_Seedling; 
        
        # calculate the number of vegetated cells after each time step 
        Cell_Plant=sum(Cells); 
        image(x=1:n1, y=1:n2, t(Cells), useRaster=T,  col = c('red','green'), axes = FALSE,
              xlab='Distance (m)', ylab='Distance (m)')
        axis(1, at = seq(0, n1, by = 250), labels = c('0','25','50','75','100'))
        axis(2, at = seq(0, n2, by = 250), labels = c('0','25','50','75','100'))
        box()
        title(main = "Revegetation of mudflats", font.main = 4)
        # The following two lines prevent flickering of the screen
        dev.flush() # Force the model to update the graphs
        dev.hold()  # Put all updating on hold  
        
}

